@@ -1,89 +1,91 @@
 package Net::DNS::RR::APL;
 
-#
-# $Id: APL.pm 1382 2015-08-26 20:39:33Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
-
-
 use strict;
+use warnings;
+our $VERSION = (qw$Id: APL.pm 2003 2025-01-21 12:06:06Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::APL - DNS APL resource record
 
 =cut
 
-
 use integer;
 
 use Carp;
 
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
-	my $self = shift;
-	my ( $data, $offset ) = @_;
+	my ( $self, $data, $offset ) = @_;
 
 	my $limit = $offset + $self->{rdlength};
 
-	$self->{aplist} = [];
+	my $aplist = $self->{aplist} = [];
 	while ( $offset < $limit ) {
 		my $xlen = unpack "\@$offset x3 C", $$data;
 		my $size = ( $xlen & 0x7F );
 		my $item = bless {}, 'Net::DNS::RR::APL::Item';
-		$item->negate(1) if $xlen & 0x80;
+		$item->{negate} = $xlen - $size;
 		@{$item}{qw(family prefix address)} = unpack "\@$offset n C x a$size", $$data;
 		$offset += $size + 4;
-		push @{$self->{aplist}}, $item;
+		push @$aplist, $item;
 	}
 	croak('corrupt APL data') unless $offset == $limit;	# more or less FUBAR
+	return;
 }
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	my $rdata = '';
-	return $rdata unless $self->{aplist};
-	foreach ( @{$self->{aplist}} ) {
+	my @rdata;
+	my $aplist = $self->{aplist};
+	foreach (@$aplist) {
 		my $address = $_->{address};
-		my $xlength = $_->negate | length($address);
-		$rdata .= pack 'n C2 a*', @{$_}{qw(family prefix)}, $xlength, $address;
+		$address =~ s/[\000]+$//;			# strip trailing null octets
+		my $xlength = ( $_->{negate} ? 0x80 : 0 ) | length($address);
+		push @rdata, pack 'n C2 a*', @{$_}{qw(family prefix)}, $xlength, $address;
 	}
-	return $rdata;
+	return join '', @rdata;
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{aplist};
-	my @rdata = map $_->string, @{$self->{aplist}};
+	my $aplist = $self->{aplist};
+	my @rdata  = map { $_->string } @$aplist;
+	return @rdata;
 }
 
 
 sub _parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
-	$self->aplist(@_);
+	$self->aplist(@argument);
+	return;
 }
 
 
 sub aplist {
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
-	while ( scalar @_ ) {					# parse apitem strings
-		last unless $_[0] =~ m#^(!?)(\d+):(.+)/(\d+)$#;
+	while ( scalar @argument ) {				# parse apitem strings
+		last unless $argument[0] =~ m#[!:./]#;
+		local $_ = shift @argument;
+		m#^(!?)(\d+):(.+)/(\d+)$#;
 		my $n = $1 ? 1 : 0;
-		my $f = $2;
-		my ( $x, $a, $p ) = split m#^[^:]+:|/#, shift;
+		my $f = $2 || 0;
+		my $a = $3;
+		my $p = $4 || 0;
 		$self->aplist( negate => $n, family => $f, address => $a, prefix => $p );
 	}
 
 	my $aplist = $self->{aplist} ||= [];
-	if ( my %argval = @_ ) {				# parse attribute=value list
+	if ( my %argval = @argument ) {				# parse attribute=value list
 		my $item = bless {}, 'Net::DNS::RR::APL::Item';
 		while ( my ( $attribute, $value ) = each %argval ) {
 			$item->$attribute($value) unless $attribute eq 'address';
@@ -93,93 +95,59 @@ sub aplist {
 	}
 
 	my @ap = @$aplist;
-	return wantarray ? @ap : join ' ', map $_->string, @ap if defined wantarray;
+	return unless defined wantarray;
+	return wantarray ? @ap : join ' ', map { $_->string } @ap;
 }
 
 
 ########################################
 
 
-package Net::DNS::RR::APL::Item;
+package Net::DNS::RR::APL::Item;	## no critic ProhibitMultiplePackages
+
+use Net::DNS::RR::A;
+use Net::DNS::RR::AAAA;
+
+my %family = qw(1 Net::DNS::RR::A	2 Net::DNS::RR::AAAA);
 
 
 sub negate {
-	my $bit = 0x80;
-	for ( shift->{negate} ) {
-		my $set = $bit | ( $_ ||= 0 );
-		return $bit & $_ unless scalar @_;
-		$_ = (shift) ? $set : ( $set ^ $bit );
-		return $_ & $bit;
-	}
+	my ( $self, @value ) = @_;
+	for (@value) { return $self->{negate} = $_ }
+	return $self->{negate};
 }
 
 
 sub family {
-	my $self = shift;
-
-	$self->{family} = 0 + shift if scalar @_;
-	$self->{family} || 0;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{family} = 0 + $_ }
+	return $self->{family} || 0;
 }
 
 
 sub prefix {
-	my $self = shift;
-
-	$self->{prefix} = 0 + shift if scalar @_;
-	$self->{prefix} || 0;
-}
-
-
-{
-	use Net::DNS::RR::A;
-	use Net::DNS::RR::AAAA;
-
-	sub _address_1 {
-		my $self = shift;
-
-		my $A = bless {%$self}, 'Net::DNS::RR::A';
-
-		return $A->address unless scalar @_;
-
-		my $alength = ( $self->prefix + 7 ) >> 3;	# mask non-prefix bits, suppress nulls
-		my @address = unpack "C$alength", $A->address(shift);
-		my $bitmask = 0xFF << ( 8 - $self->prefix & 7 );
-		push @address, ( $bitmask & pop(@address) ) if $alength;
-		for ( reverse @address ) { last if $_; pop @address }
-		$self->{address} = pack 'C*', @address;
-	}
-
-
-	sub _address_2 {
-		my $self = shift;
-
-		my $AAAA = bless {%$self}, 'Net::DNS::RR::AAAA';
-
-		return $AAAA->address_long unless scalar @_;
-
-		my $alength = ( $self->prefix + 7 ) >> 3;	# mask non-prefix bits, suppress nulls
-		my @address = unpack "C$alength", $AAAA->address(shift);
-		my $bitmask = 0xFF << ( 8 - $self->prefix & 7 );
-		push @address, ( $bitmask & pop(@address) ) if $alength;
-		for ( reverse @address ) { last if $_; pop @address }
-		$self->{address} = pack 'C*', @address;
-	}
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{prefix} = 0 + $_ }
+	return $self->{prefix} || 0;
 }
 
 
 sub address {
-	for ( $_[0]->family ) {
-		return &_address_1 if /1/;
-		return &_address_2 if /2/;
-		die 'unknown address family';
-	}
+	my ( $self, @value ) = @_;
+
+	my $family = $family{$self->family} || die 'unknown address family';
+	return bless( {%$self}, $family )->address unless scalar @value;
+
+	my $bitmask = $self->prefix;
+	my $address = bless( {}, $family )->address( shift @value );
+	return $self->{address} = pack "B$bitmask", unpack 'B*', $address;
 }
 
 
 sub string {
 	my $self = shift;
 
-	my $not = $self->negate ? '!' : '';
+	my $not = $self->{negate} ? '!' : '';
 	my ( $family, $address, $prefix ) = ( $self->family, $self->address, $self->prefix );
 	return "$not$family:$address/$prefix";
 }
@@ -191,8 +159,8 @@ __END__
 
 =head1 SYNOPSIS
 
-    use Net::DNS;
-    $rr = new Net::DNS::RR('name IN APL aplist');
+	use Net::DNS;
+	$rr = Net::DNS::RR->new('name IN APL aplist');
 
 =head1 DESCRIPTION
 
@@ -210,17 +178,18 @@ other unpredictable behaviour.
 
 =head2 aplist
 
-    @aplist = $rr->aplist;
-  
-    @aplist = $rr->aplist( '1:192.168.32.0/21', '!1:192.168.38.0/28' );
-  
-    @aplist = $rr->aplist( '1:224.0.0.0/4', '2:FF00:0:0:0:0:0:0:0/8' );
-  
-    @aplist = $rr->aplist( negate  => 1,
-			   family  => 1,
-			   address => '192.168.38.0',
-			   prefix  => 28,
-			   );
+	@aplist = $rr->aplist;
+	
+	@aplist = $rr->aplist( '1:192.168.32.0/21', '!1:192.168.38.0/28' );
+	
+	@aplist = $rr->aplist( '1:224.0.0.0/4', '2:FF00:0:0:0:0:0:0:0/8' );
+	
+	@aplist = $rr->aplist(
+			negate	=> 1,
+			family	=> 1,
+			address => '192.168.38.0',
+			prefix	=> 28,
+			);
 
 Ordered, possibly empty, list of address prefix items.
 Additional items, if present, are appended to the existing list
@@ -235,38 +204,38 @@ created it.
 
 =head2 negate
 
-    $rr->negate(1);
+	$rr->negate(1);
 
-    if ( $rr->negate ) {
-	...
-    }
+	if ( $rr->negate ) {
+		...
+	}
 
 Boolean attribute indicating the prefix to be an address range exclusion.
 
 =head2 family
 
-    $family = $rr->family;
-    $rr->family( $family );
+	$family = $rr->family;
+	$rr->family( $family );
 
 Address family discriminant.
 
 =head2 prefix
 
-    $prefix = $rr->prefix;
-    $rr->prefix( $prefix );
+	$prefix = $rr->prefix;
+	$rr->prefix( $prefix );
 
 Number of bits comprising the address prefix.
 
 
 =head2 address
 
-    $address = $object->address;
+	$address = $object->address;
 
 Address portion of the prefix list item.
 
 =head2 string
 
-    $string = $object->string;
+	$string = $object->string;
 
 Returns the prefix list item in the form required in zone files.
 
@@ -275,7 +244,7 @@ Returns the prefix list item in the form required in zone files.
 
 Copyright (c)2008 Olaf Kolkman, NLnet Labs.
 
-Portions Copyright (c)2011 Dick Franks.
+Portions Copyright (c)2011,2017 Dick Franks.
 
 All rights reserved.
 
@@ -286,7 +255,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -303,6 +272,7 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC3123
+L<perl> L<Net::DNS> L<Net::DNS::RR>
+L<RFC3123|https://iana.org/go/rfc3123>
 
 =cut
